function [z] = integrate_with_prior(p,q,z0,lambda,mask)
% Merges rough depth prior with high precision estimation of gradient.
% Solves the variational problem :
%  min   \iint_{(u,v) \in mask} [\nabla z(u,v)-[p(u,v),q(u,v)]^\top ]^2 du dv
%   z                           + \lambda [z(u,v)-z0(u,v)]^2 du dv
%
% 
% Inputs :
% * p : nrows x ncols : approximation of \partial_u z
% * q : nrows x ncols : approximation of \partial_v z
% * z0 : nrows x ncols : depth prior (with NaN values when no prior available)
% * lambda : 1x1, or nrows x ncols : prior weight
% * mask : nrows x ncols : binary mask
%
% Outputs :
% * z : nrows x ncols : integrated result
% * b : card(mask) x 1 vector
%
% Usage :
% > lambda = 1e-3; % For instance
% > z0 = zeros(size(mask)); % For instance
% > [z] = integrate_with_prior(p,q,z0,lambda,mask);
%
% Ref : Yvain Qu\'eau - Reconstruction tridimensionnelle par stéréophotométrie
%       Chapter 2
%       Ph.D Thesis - Universit\'e de Toulouse
	if(nargin<2)
		disp('Not enough inputs');
		return;
	end
	if (~exist('mask','var')||isempty(mask)), mask=ones(size(p));end
	if (~exist('lambda','var')||isempty(lambda)), lambda=1e-9;end
	if (~exist('z0','var')||isempty(z0)), z0=zeros(size(mask));end

	z0(mask==0) = NaN;
	indices_mask = find(mask>0);

	% Make the weights spatially-varying
	if(length(lambda)==1)
		lambda_old = lambda;
		lambda = zeros(size(mask));
		lambda(indices_mask) = lambda_old;
	end 

	% Look for missing prior data
	indices_missing_prior = find(isnan(z0));
	indices_prior = find(~isnan(z0));
	lambda(indices_missing_prior) = 0;

	% Create integration system (without prior)
	[A,b,mapping_matrix] = make_Integration_system(p,q,mask);

	% Modify A and b to include the prior
	if(length(indices_prior)==0)
		A = A+1e-9*speye(size(A));
	elseif(length(indices_prior)==length(indices_mask))
		A = A+bsxfun(@times,lambda(indices_prior),speye(size(A)));
		b = b+lambda(indices_prior).*z0(indices_prior);
	else
		for px = 1:length(indices_prior)
			el = mapping_matrix(indices_prior(px)); % Number of the pixels within the masked pixels
			A(el,el) = A(el,el) + lambda(indices_prior(px));
			b(el) = b(el) + lambda(indices_prior(px))*z0(indices_prior(px));
		end
	end

	% Solve : A is positive symmetric definite -> \ is Cholesky (n2 cost)
	z = z0;
	z(indices_mask) = A\b;
end
