function [A,B,mapping_matrix,Omega] = make_Poisson_system(p,q,mask)
% Creates the linear system associated with variational problem :
%  min   \iint_{(u,v) \in mask} [\nabla z(u,v)-[p(u,v),q(u,v)]^\top ]^2 du dv
%   z
% with natural boundary condition (\nabla z - [p,q]^\top) \dot \mu = 0
%
% The variational problem is turned into the discrete minimization problem
%  min      \sum_{(u+1,v) \in \Omega} [z(u+1,v)-z(u,v)-pbar(u,v)]^2 
% {z(u,v)}  + \sum_{(u,v+1) \in \Omega} [z(u,v+1)-z(u,v)-qbar(u,v)]^2 
%  with pbar(u,v) = 0.5*(p(u,v)+p(u+1,v)) and qbar(u,v) = 0.5*(q(u,v)+q(u+1,v))
%
% Writing the optimality condition associated with each z(u,v) provides a linear
% system Az = b in the elements of z
% * A is a symmetric, semi-definite matrix with rank-1 deficiency (Laplacian matrix)
% * b contains the approximate divergence of (p,q)
%
% Inputs :
% * p : nrows x ncols : approximation of \partial_u z
% * q : nrows x ncols : approximation of \partial_v z
% * mask : nrows x ncols : binary mask
%
% Outputs :
% * A : card(mask) x card(mask) matrix
% * B : card(mask) x 1 vector
%
% Usage :
% > [A,b] = make_Poisson_system(p,q,mask);
% > z = zeros(size(mask));
% > z(find(mask>0)) = A\b; % Or any faster solver...
%
% Ref : Yvain Qu\'eau - Reconstruction tridimensionnelle par stéréophotométrie
%       Chapter 2
%       Ph.D Thesis - Universit\'e de Toulouse

	if(nargin < 3)
		mask = ones(size(mask));
	end

	% Calculate some usefuk masks
	Omega = zeros(size(mask,1),size(mask,2),4);
	Omega_padded = padarray(mask,[1 1],0);
	Omega(:,:,1) = Omega_padded(3:end,2:end-1).*mask; % In mask, bottom neighbor in mask	
	Omega(:,:,2) = Omega_padded(1:end-2,2:end-1).*mask; % In mask, top neihbor in mask
	Omega(:,:,3) = Omega_padded(2:end-1,3:end).*mask; % In mask, right neighbor in mask	
	Omega(:,:,4) = Omega_padded(2:end-1,1:end-2).*mask; % In mask, left neihbor in mask
	clear Omega_padded 	

	% Mapping
	indices_mask = find(mask>0);
	mapping_matrix = zeros(size(p));
	mapping_matrix(indices_mask)=1:length(indices_mask);
	pbar = 0.5*(p+p([2:end end],:));
	qbar = 0.5*(q+q(:,[2:end end]));

	% System
	I = [];
	J = [];
	K = [];
	B = zeros(length(indices_mask),1);

	% In mask, right neighbor in mask
	set = Omega(:,:,3);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X,Y+1);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = ones(length(indices_centre),1);
	A_voisin = -ones(length(indices_centre),1);	
	K=[K;A_centre(:);A_voisin(:)]; % Which values ?
	I=[I;I_centre(:);I_centre(:)]; % Which values ?
	J=[J;I_centre(:);I_voisins(:)]; % Which values ?
	B(I_centre) = B(I_centre)-qbar(indices_centre);
	
	
	% In mask, left neighbor in mask
	set = Omega(:,:,4);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X,Y-1);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = ones(length(indices_centre),1);
	A_voisin = -ones(length(indices_centre),1);	
	K=[K;A_centre(:);A_voisin(:)]; % Which values ?
	I=[I;I_centre(:);I_centre(:)]; % Which values ?
	J=[J;I_centre(:);I_voisins(:)]; % Which values ?
	B(I_centre) = B(I_centre)+qbar(indices_voisins);
	
	% In mask, top neighbor in mask
	set = Omega(:,:,2);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X-1,Y);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = ones(length(indices_centre),1);
	A_voisin = -ones(length(indices_centre),1);		
	K=[K;A_centre(:);A_voisin(:)]; % Which values ?
	I=[I;I_centre(:);I_centre(:)]; % Which values ?
	J=[J;I_centre(:);I_voisins(:)]; % Which values ?
	B(I_centre) = B(I_centre)+pbar(indices_voisins);	
		
	% In mask, bottom neighbor in mask
	set = Omega(:,:,1);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X+1,Y);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = ones(length(indices_centre),1);
	A_voisin = -ones(length(indices_centre),1);	
	K=[K;A_centre(:);A_voisin(:)]; % Which values ?
	I=[I;I_centre(:);I_centre(:)]; % Which values ?
	J=[J;I_centre(:);I_voisins(:)]; % Which values ?
	B(I_centre) = B(I_centre)-pbar(indices_centre);	
	
	% Construction de A : 
	A = sparse(I,J,K);

end
